/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

/* [] END OF FILE */
#include <project.h>
#include <BNO080.h>
#include <stdio.h>
#include <stdlib.h>

#include <stdbool.h>


//Registers
uint8_t CHANNEL_EXECUTABLE = 1;
uint8_t CHANNEL_CONTROL = 2;
uint8_t CHANNEL_REPORTS = 3;


//These are used for low level communication with the sensor, on channel 2

uint8 IMU_Address = 0x4B;

//Global Variables
uint8_t shtpHeader[4];
uint8_t shtpData[MAX_PACKET_SIZE];
uint8_t sequenceNumber[6] = {0, 0, 0, 0, 0, 0};
uint8_t commandSequenceNumber = 0;
uint32 metaData[MAX_METADATA_SIZE];

//Q values 
 int16_t rotationVector_Q1 = 14;
 int16_t accelerometer_Q1 = 8;
 int16_t linear_accelerometer_Q1 = 8;
 int16_t gyro_Q1 = 9;
 int16_t magnetometer_Q1 = 4;

//Local Variables
uint16_t rawAccelX, rawAccelY, rawAccelZ, accelAccuracy;
uint16_t rawLinAccelX, rawLinAccelY, rawLinAccelZ, accelLinAccuracy;
uint16_t rawGyroX, rawGyroY, rawGyroZ, gyroAccuracy;
uint16_t rawMagX, rawMagY, rawMagZ, magAccuracy;
uint16_t rawQuatI, rawQuatJ, rawQuatK, rawQuatReal, rawQuatRadianAccuracy, quatAccuracy;
uint16_t stepCount;
uint32_t timeStamp;
uint8_t stabilityClassifier;
uint8_t activityClassifier;
uint8_t *_activityConfidences;						  //Array that store the confidences of the 9 possible activities
uint8_t calibrationStatus;							  //Byte R0 of ME Calibration Response
uint16_t memsRawAccelX, memsRawAccelY, memsRawAccelZ; //Raw readings from MEMS sensor
uint16_t memsRawGyroX, memsRawGyroY, memsRawGyroZ;	//Raw readings from MEMS sensor
uint16_t memsRawMagX, memsRawMagY, memsRawMagZ;		  //Raw readings from MEMS sensor



//BNO080 FUNCTIONS
void BNO080_sendPacket(uint8_t channelNumber, uint8_t dataLength) {
    UART_PutString("Beginning of send Packet sequence\n");
    uint8_t packetLength = dataLength+4; // adding the 4 header bytes
    //debugging tool
    char Buff[20];
    sprintf(Buff,"d=%d\n", packetLength);
    
    I2C_MasterSendStart(IMU_Address, I2C_WRITE_XFER_MODE);
    I2C_MasterWriteByte(packetLength);
    UART_PutString("First write command sent\n");
    I2C_MasterWriteByte(packetLength >> 8);
    UART_PutString("Second write command sent\n");
    I2C_MasterWriteByte(channelNumber);
    UART_PutString("Third write command sent\n");
    I2C_MasterWriteByte(sequenceNumber[channelNumber]++);
    
    //Sending the User's data packet
    for (uint8_t i=0; i < dataLength; i++){
        I2C_MasterWriteByte(shtpData[i]);
    }
    I2C_MasterSendStop();
    UART_PutString("Packet sent\n");
    
    //Add some form of error check here
}

void BNO080_begin(uint8_t deviceAddress){
    
    UART_PutString("entered begin sequence\n");
    BNO080_softReset();
    
    IMU_Address = deviceAddress;
    
    shtpData[0] = SHTP_REPORT_PRODUCT_ID_REQUEST;
    shtpData[1]= 0;
    
    BNO080_sendPacket(CHANNEL_CONTROL,2);
}

bool BNO080_receivePacket() {
    //Master send stop
    I2C_MasterSendStart(IMU_Address, I2C_READ_XFER_MODE);
    uint8_t packetLSB = I2C_MasterReadByte(I2C_ACK_DATA);
    uint8_t packetMSB = I2C_MasterReadByte(I2C_ACK_DATA);
    uint8_t channelNumber = I2C_MasterReadByte(I2C_ACK_DATA);
    uint8_t sequenceNumber = I2C_MasterReadByte(I2C_NAK_DATA);
    I2C_MasterSendStop();
    
    //Assign header Bytes
    shtpHeader[0] = packetLSB;
    shtpHeader[1] = packetMSB;
    shtpHeader[2] = channelNumber;
    shtpHeader[3] = sequenceNumber;

    //Calculating number of bytes to read
    uint16_t dataLength = ((uint16_t)packetMSB << 8 | packetLSB);
    dataLength &= ~(1<<15);
    if (dataLength == 0) {
       UART_PutString("Not Receiving Data\n");
       // sprintf("Not receiving Data\n");
       return false;
    }
   
    char Buff[20];
    sprintf(Buff,"p=%d\n",dataLength);
    UART_PutString(Buff);
    
    BNO080_getData(dataLength);
    return true;
    
}

void BNO080_softReset() {
    shtpData[0] = 1;
    
    BNO080_sendPacket(CHANNEL_EXECUTABLE, 1);
    CyDelay(50);
    
    //flush incoming data
    while (BNO080_receivePacket() == true) {
        UART_PutString("receiving data");
        CyDelay(50);
    }
        
    
    
}

void BNO080_getData(uint16_t bytesRemaining) {
   uint16_t dataSpot = 0;
   
    //Read the bytes
    I2C_MasterSendStart(IMU_Address, I2C_READ_XFER_MODE);
   while (bytesRemaining > 0) {
    uint16_t numberOffBytesToRead = bytesRemaining;
    //Add error check for buffer length
    
    //first 4 bytes are header bytes **is there a way to check how this works?**
    for (uint8_t j =0; j < 4; j++){
       I2C_MasterReadByte(I2C_ACK_DATA);
    }
    //Read the rest of the bytes
    for (dataSpot=0; dataSpot < numberOffBytesToRead; dataSpot++) {
        uint8_t incomingData = I2C_MasterReadByte(I2C_ACK_DATA);
        shtpData[dataSpot] = incomingData;       
    }
    bytesRemaining -= numberOffBytesToRead;
    }
    I2C_MasterSendStop();
}

void BNO080_setFeatureCommand(uint8_t reportID, uint16_t timeBetweenReports) {
    //time input in miliseconds
    long microsBetweenReports = (long)timeBetweenReports * 1000L;
    
    shtpData[0] = SHTP_REPORT_SET_FEATURE_COMMAND;
    shtpData[1] = reportID;
    shtpData[2] = 0;
    shtpData[3] = 0;
    shtpData[4] = 0;
    shtpData[5] = (microsBetweenReports >> 0) & 0xFF;
    shtpData[6] = (microsBetweenReports >> 8) & 0xFF;
    shtpData[7] = (microsBetweenReports >> 16) & 0xFF;
    shtpData[8] = (microsBetweenReports >> 24) & 0xFF;
    shtpData[9] = 0;
    shtpData[10] = 0;
    shtpData[11] = 0; 
    shtpData[12] = 0;
    shtpData[13] = (0 >> 0) & 0xFF;
    shtpData[14] = (0 >> 8) & 0xFF;
    shtpData[15] = (0 >> 16) & 0xFF;
    shtpData[16] = (0 >> 24) & 0xFF; 
    
    BNO080_sendPacket(CHANNEL_CONTROL, 17);
}

void BNO080_dataAvailable() {
    BNO080_receivePacket();
    
    if (shtpHeader[2] == CHANNEL_REPORTS && shtpData[0] == SHTP_REPORT_BASE_TIMESTAMP) {
        parseInputReport();
        return;
    }
    else if (shtpHeader[2] == CHANNEL_CONTROL)
    {
        parseCommandReport();
        return;
    }
}
void parseCommandReport() {
    if (shtpData[0] == SHTP_REPORT_COMMAND_RESPONSE) {
        uint8_t command = shtpData[2];
        if (command == COMMAND_ME_CALIBRATE) {
            calibrationStatus = shtpData[5+0];
        }
    }

}

void parseInputReport () {
    
    uint16_t dataLength = ((uint16_t) shtpHeader[1] << 8 | shtpHeader[0]);
    dataLength &= ~(1 << 15);   
    dataLength -= 4; //removing the header data
    
    timeStamp = ((uint32_t)shtpData[4]<<(8*3))|((uint32_t)shtpData[2]<<(8*1))|((uint32_t)shtpData[1]<<(8*0));
    
    uint8_t status = shtpData[5 + 2] & 0x03;
    uint16_t data1 = (uint16_t) shtpData[5+5] << 8 | shtpData[5+4];
    uint16_t data2 = (uint16_t) shtpData[5+7] << 8 | shtpData[5+6];
    uint16_t data3 = (uint16_t) shtpData[5+9] << 8 | shtpData[5+8];
    uint16_t data4 = 0;
    uint16_t data5 = 0;
    
    if (dataLength - 5 > 9) {
        data4 = (uint16_t)shtpData[5+11] << 8 | shtpData[5+10];
    }
    if (dataLength - 5 > 11) {
        data5 = (uint16_t)shtpData[5+13] << 8 | shtpData[5 +12];
        
    }
    
    //Storing the data to their respective variables
    if (shtpData[5] == SENSOR_REPORTID_ACCELEROMETER){
        accelAccuracy = status;
        rawAccelX = data1;
        rawAccelY = data2;
        rawAccelZ = data3;
    }
    else if (shtpData[5] == SENSOR_REPORTID_LINEAR_ACCELERATION) {
        accelLinAccuracy = status;
        rawLinAccelX = data1;
        rawLinAccelY = data2;
        rawLinAccelZ = data3;
    }
    
    else if (shtpData[5] == SENSOR_REPORTID_GYROSCOPE) {
        gyroAccuracy = status;
        rawGyroX = data1;
        rawGyroY = data2;
        rawGyroZ = data3;
        
    }
     
}

float BNO080_qToFloat(uint16_t fixedPointValue, uint8_t qPoint) {
    float qFloat = fixedPointValue;
    qFloat *= pow(2, qPoint*-1);
    return (qFloat);
} 

float BNO080_getAccelX(){
    float accel = BNO080_qToFloat(rawAccelX, accelerometer_Q1);
    return (accel);
}




void BNO080_enableRotationVector(uint16_t timeBetweenReports) {
    BNO080_setFeatureCommand(SENSOR_REPORTID_ACCELEROMETER, timeBetweenReports); //remember to change this back to reportid_rotation_vector
}
