/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include <project.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <stdbool.h>


//Registers
extern uint8_t CHANNEL_EXECUTABLE;
extern uint8_t CHANNEL_CONTROL;
extern uint8_t CHANNEL_REPORTS;

#define SHTP_REPORT_COMMAND_RESPONSE 0xF1
#define SHTP_REPORT_PRODUCT_ID_REQUEST 0xF9
#define SHTP_REPORT_BASE_TIMESTAMP 0xFB
#define SHTP_REPORT_SET_FEATURE_COMMAND 0xFD

//
#define SENSOR_REPORTID_ACCELEROMETER 0x01
#define SENSOR_REPORTID_GYROSCOPE 0x02
#define SENSOR_REPORTID_MAGNETIC_FIELD 0x03
#define SENSOR_REPORTID_LINEAR_ACCELERATION 0x04
#define SENSOR_REPORTID_ROTATION_VECTOR 0x05
#define SENSOR_REPORTID_GRAVITY 0x06
#define SENSOR_REPORTID_GAME_ROTATION_VECTOR 0x08
#define SENSOR_REPORTID_GEOMAGNETIC_ROTATION_VECTOR 0x09
#define SENSOR_REPORTID_TAP_DETECTOR 0x10
#define SENSOR_REPORTID_STEP_COUNTER 0x11
#define SENSOR_REPORTID_STABILITY_CLASSIFIER 0x13
#define SENSOR_REPORTID_RAW_ACCELEROMETER 0x14
#define SENSOR_REPORTID_RAW_GYROSCOPE 0x15
#define SENSOR_REPORTID_RAW_MAGNETOMETER 0x16
#define SENSOR_REPORTID_PERSONAL_ACTIVITY_CLASSIFIER 0x1E


#define MAX_PACKET_SIZE 128
#define MAX_METADATA_SIZE 9

//Global Variables
extern uint8_t shtpHeader[4];
extern uint8_t shtpData[MAX_PACKET_SIZE];
extern uint8_t sequenceNumber[6]; 
extern uint8_t commandSequenceNumber;
extern uint32 metaData[MAX_METADATA_SIZE];

//Command IDs
#define COMMAND_ME_CALIBRATE 7


//Functions
void BNO080_softReset();
void BNO080_begin(uint8 deviceAddress);
void BNO080_sendPacket(uint8 channelNumber, uint8 dataLength);
bool BNO080_receivePacket();
void BNO080_getData(uint16_t bytesRemaining);
void BNO080_setFeatureCommand(uint8_t reportID, uint16_t timeBetweenReports);
void BNO080_enableRotationVector(uint16_t timeBetweenReports);
void parseCommandReport();
void parseInputReport ();
float BNO080_qToFloat(uint16_t fixedPointValue, uint8_t qPoint);
float BNO080_getAccelX();


//Some local variables 
extern uint16_t rawAccelX, rawAccelY, rawAccelZ, accelAccuracy;
extern uint16_t rawLinAccelX, rawLinAccelY, rawLinAccelZ, accelLinAccuracy;
extern uint16_t rawGyroX, rawGyroY, rawGyroZ, gyroAccuracy;
extern uint16_t rawMagX, rawMagY, rawMagZ, magAccuracy;
extern uint16_t rawQuatI, rawQuatJ, rawQuatK, rawQuatReal, rawQuatRadianAccuracy, quatAccuracy;
extern uint16_t stepCount;
extern uint32_t timeStamp;
extern uint8_t stabilityClassifier;
extern uint8_t activityClassifier;
extern uint8_t *_activityConfidences;						  //Array that store the confidences of the 9 possible activities
extern uint8_t calibrationStatus;							  //Byte R0 of ME Calibration Response
extern uint16_t memsRawAccelX, memsRawAccelY, memsRawAccelZ; //Raw readings from MEMS sensor
extern uint16_t memsRawGyroX, memsRawGyroY, memsRawGyroZ;	//Raw readings from MEMS sensor
extern uint16_t memsRawMagX, memsRawMagY, memsRawMagZ;		  //Raw readings from MEMS sensor

//Q values from datasheet
extern int16_t rotationVector_Q1;
extern int16_t accelerometer_Q1;
extern int16_t linear_accelerometer_Q1;
extern int16_t gyro_Q1;
extern int16_t magnetometer_Q1;


/* [] END OF FILE */
